@@ -22,8 +22,8 @@
  */
 
 
-#ifndef __NDPI_PROTOCOLS_IDS_H__
-#define __NDPI_PROTOCOLS_IDS_H__
+#ifndef __NDPI_PROTOCOL_IDS_H__
+#define __NDPI_PROTOCOL_IDS_H__
 
 #define NDPI_PROTOCOL_SIZE                  2
 
@@ -60,7 +60,7 @@ typedef enum {
   NDPI_PROTOCOL_MAIL_SMTPS            = 29,
   NDPI_PROTOCOL_DTLS                  = 30,
   NDPI_PROTOCOL_UBNTAC2               = 31, /* Ubiquity UBNT AirControl = 2 */
-  NDPI_PROTOCOL_KONTIKI               = 32,
+  NDPI_PROTOCOL_BFCP                  = 32,
   NDPI_PROTOCOL_YANDEX_MAIL           = 33,
   NDPI_PROTOCOL_YANDEX_MUSIC          = 34,
   NDPI_PROTOCOL_GNUTELLA              = 35,
@@ -82,12 +82,12 @@ typedef enum {
   NDPI_PROTOCOL_MAIL_IMAPS            = 51,
   NDPI_PROTOCOL_ICECAST               = 52,
   NDPI_PROTOCOL_CPHA                  = 53,
-  NDPI_PROTOCOL_PPSTREAM              = 54,
+  NDPI_PROTOCOL_IQIYI                 = 54,
   NDPI_PROTOCOL_ZATTOO                = 55,
   NDPI_PROTOCOL_YANDEX_MARKET         = 56,
   NDPI_PROTOCOL_YANDEX_DISK           = 57,
   NDPI_PROTOCOL_DISCORD               = 58,
-  NDPI_PROTOCOL_TVUPLAYER             = 59,
+  NDPI_PROTOCOL_ADOBE_CONNECT         = 59,
   NDPI_PROTOCOL_MONGODB               = 60,
   NDPI_PROTOCOL_PLURALSIGHT           = 61,
   NDPI_PROTOCOL_YANDEX_CLOUD          = 62,
@@ -100,7 +100,7 @@ typedef enum {
   NDPI_PROTOCOL_AMONG_US              = 69,
   NDPI_PROTOCOL_YAHOO                 = 70,
   NDPI_PROTOCOL_DISNEYPLUS            = 71,
-  NDPI_PROTOCOL_GOOGLE_PLUS           = 72,
+  NDPI_PROTOCOL_HART_IP               = 72,
   NDPI_PROTOCOL_IP_VRRP               = 73,
   NDPI_PROTOCOL_STEAM                 = 74,
   NDPI_PROTOCOL_HALFLIFE2             = 75,
@@ -155,7 +155,7 @@ typedef enum {
   NDPI_PROTOCOL_YOUTUBE               = 124,
   NDPI_PROTOCOL_SKYPE_TEAMS           = 125,
   NDPI_PROTOCOL_GOOGLE                = 126,
-  NDPI_PROTOCOL_RPC                   = 127,
+  NDPI_PROTOCOL_MS_RPCH               = 127,
   NDPI_PROTOCOL_NETFLOW               = 128,
   NDPI_PROTOCOL_SFLOW                 = 129,
   NDPI_PROTOCOL_HTTP_CONNECT          = 130,
@@ -177,7 +177,7 @@ typedef enum {
   NDPI_PROTOCOL_RADIUS                = 146,
   NDPI_PROTOCOL_WINDOWS_UPDATE        = 147,
   NDPI_PROTOCOL_TEAMVIEWER            = 148,
-  NDPI_PROTOCOL_TUENTI                = 149,
+  NDPI_PROTOCOL_EGD                   = 149,
   NDPI_PROTOCOL_LOTUS_NOTES           = 150,
   NDPI_PROTOCOL_SAP                   = 151,
   NDPI_PROTOCOL_GTP                   = 152,
@@ -185,7 +185,7 @@ typedef enum {
   NDPI_PROTOCOL_LLMNR                 = 154,
   NDPI_PROTOCOL_TOCA_BOCA             = 155,
   NDPI_PROTOCOL_SPOTIFY               = 156,
-  NDPI_PROTOCOL_MESSENGER             = 157,
+  NDPI_PROTOCOL_FACEBOOK_MESSENGER    = 157,
   NDPI_PROTOCOL_H323                  = 158,
   NDPI_PROTOCOL_OPENVPN               = 159,
   NDPI_PROTOCOL_NOE                   = 160,
@@ -210,11 +210,11 @@ typedef enum {
   NDPI_PROTOCOL_EBAY                  = 179,
   NDPI_PROTOCOL_CNN                   = 180,
   NDPI_PROTOCOL_MEGACO                = 181,
-  NDPI_PROTOCOL_REDIS                 = 182,
+  NDPI_PROTOCOL_RESP                  = 182,
   NDPI_PROTOCOL_PINTEREST             = 183,
   NDPI_PROTOCOL_VHUA                  = 184,
   NDPI_PROTOCOL_TELEGRAM              = 185,
-  NDPI_PROTOCOL_VEVO                  = 186,
+  NDPI_PROTOCOL_COD_MOBILE            = 186,
   NDPI_PROTOCOL_PANDORA               = 187,
   NDPI_PROTOCOL_QUIC                  = 188,
   NDPI_PROTOCOL_ZOOM                  = 189, /* Zoom video conference. */
@@ -229,7 +229,7 @@ typedef enum {
   NDPI_PROTOCOL_MPEGTS                = 198,
   NDPI_PROTOCOL_SNAPCHAT              = 199,
   NDPI_PROTOCOL_SINA                  = 200,
-  NDPI_PROTOCOL_HANGOUT_DUO           = 201, /* Google Hangout ad Duo (merged as they are very similar) */
+  NDPI_PROTOCOL_GOOGLE_MEET           = 201,
   NDPI_PROTOCOL_IFLIX                 = 202,
   NDPI_PROTOCOL_GITHUB                = 203,
   NDPI_PROTOCOL_BJNP                  = 204,
@@ -263,7 +263,7 @@ typedef enum {
   NDPI_PROTOCOL_PASTEBIN              = 232,
   NDPI_PROTOCOL_LINKEDIN              = 233,
   NDPI_PROTOCOL_SOUNDCLOUD            = 234,
-  NDPI_PROTOCOL_CSGO                  = 235, /* Counter-Strike Global Offensive, Dota = 2 */
+  NDPI_PROTOCOL_VALVE_SDR             = 235, /* Used by all modern Valve games */
   NDPI_PROTOCOL_LISP                  = 236,
   NDPI_PROTOCOL_DIAMETER              = 237,
   NDPI_PROTOCOL_APPLE_PUSH            = 238,
@@ -384,7 +384,91 @@ typedef enum {
   NDPI_PROTOCOL_PROTOBUF              = 353,
   NDPI_PROTOCOL_ETHEREUM              = 354, 
   NDPI_PROTOCOL_TELEGRAM_VOIP         = 355,
-  NDPI_PROTOCOL_SINA_WEIBO	      = 356,	
+  NDPI_PROTOCOL_SINA_WEIBO            = 356,
+  NDPI_PROTOCOL_TESLA_SERVICES        = 357,
+  NDPI_PROTOCOL_PTPV2                 = 358,
+  NDPI_PROTOCOL_RTPS                  = 359,
+  NDPI_PROTOCOL_OPC_UA                = 360,
+  NDPI_PROTOCOL_S7COMM_PLUS           = 361,
+  NDPI_PROTOCOL_FINS                  = 362,
+  NDPI_PROTOCOL_ETHERSIO              = 363,
+  NDPI_PROTOCOL_UMAS                  = 364,
+  NDPI_PROTOCOL_BECKHOFF_ADS          = 365,
+  NDPI_PROTOCOL_ISO9506_1_MMS         = 366,
+  NDPI_PROTOCOL_IEEE_C37118           = 367,
+  NDPI_PROTOCOL_ETHERSBUS             = 368,
+  NDPI_PROTOCOL_MONERO                = 369,
+  NDPI_PROTOCOL_DCERPC                = 370,
+  NDPI_PROTOCOL_PROFINET_IO           = 371,
+  NDPI_PROTOCOL_HISLIP                = 372,
+  NDPI_PROTOCOL_UFTP                  = 373,
+  NDPI_PROTOCOL_OPENFLOW              = 374,
+  NDPI_PROTOCOL_JSON_RPC              = 375,
+  NDPI_PROTOCOL_WEBDAV                = 376,
+  NDPI_PROTOCOL_APACHE_KAFKA          = 377,
+  NDPI_PROTOCOL_NOMACHINE             = 378,
+  NDPI_PROTOCOL_IEC62056              = 379,
+  NDPI_PROTOCOL_HL7                   = 380, /* Hospital Protocol */
+  NDPI_PROTOCOL_CEPH                  = 381,
+  NDPI_PROTOCOL_GOOGLE_CHAT           = 382,
+  NDPI_PROTOCOL_ROUGHTIME             = 383,
+  NDPI_PROTOCOL_PIA                   = 384,
+  NDPI_PROTOCOL_KCP                   = 385,
+  NDPI_PROTOCOL_DOTA2                 = 386,
+  NDPI_PROTOCOL_MUMBLE                = 387,
+  NDPI_PROTOCOL_YOJIMBO               = 388,
+  NDPI_PROTOCOL_ELECTRONICARTS        = 389,
+  NDPI_PROTOCOL_STOMP                 = 390,
+  NDPI_PROTOCOL_RADMIN                = 391,
+  NDPI_PROTOCOL_RAFT                  = 392,
+  NDPI_PROTOCOL_CIP                   = 393,
+  NDPI_PROTOCOL_GEARMAN               = 394,
+  NDPI_PROTOCOL_TENCENTGAMES          = 395,
+  NDPI_PROTOCOL_GAIJIN                = 396,
+  NDPI_PROTOCOL_C1222                 = 397,
+  NDPI_PROTOCOL_HUAWEI                = 398,
+  NDPI_PROTOCOL_HUAWEI_CLOUD          = 399,
+  NDPI_PROTOCOL_DLEP                  = 400,
+  NDPI_PROTOCOL_BFD                   = 401,
+  NDPI_PROTOCOL_NETEASE_GAMES         = 402,
+  NDPI_PROTOCOL_PATHOFEXILE           = 403,
+  NDPI_PROTOCOL_GOOGLE_CALL           = 404, /* Voip/audio/video calls from Google Chat/Meet/... */
+  NDPI_PROTOCOL_PFCP                  = 405,
+  NDPI_PROTOCOL_FLUTE                 = 406,
+  NDPI_PROTOCOL_LOLWILDRIFT           = 407,
+  NDPI_PROTOCOL_TESO                  = 408,
+  NDPI_PROTOCOL_LDP                   = 409,
+  NDPI_PROTOCOL_KNXNET_IP             = 410,
+  NDPI_PROTOCOL_BLUESKY               = 411,
+  NDPI_PROTOCOL_MASTODON              = 412,
+  NDPI_PROTOCOL_THREADS               = 413,
+  NDPI_PROTOCOL_VIBER_VOIP            = 414,
+  NDPI_PROTOCOL_ZUG                   = 415,
+  NDPI_PROTOCOL_JRMI                  = 416, 
+  NDPI_PROTOCOL_RIPE_ATLAS            = 417,
+  NDPI_PROTOCOL_HLS                   = 418,
+  NDPI_PROTOCOL_CLICKHOUSE            = 419,
+  NDPI_PROTOCOL_NANO                  = 420,
+  NDPI_PROTOCOL_OPENWIRE              = 421,
+  NDPI_PROTOCOL_CNP_IP                = 422,
+  NDPI_PROTOCOL_ATG                   = 423,
+  NDPI_PROTOCOL_TRDP                  = 424,
+  NDPI_PROTOCOL_LUSTRE                = 425,
+  NDPI_PROTOCOL_NORDVPN               = 426,
+  NDPI_PROTOCOL_SURFSHARK             = 427,
+  NDPI_PROTOCOL_CACTUSVPN             = 428,
+  NDPI_PROTOCOL_WINDSCRIBE            = 429,
+  NDPI_PROTOCOL_SONOS                 = 430,
+  NDPI_PROTOCOL_DINGTALK              = 431,
+  NDPI_PROTOCOL_PALTALK               = 432,
+  NDPI_PROTOCOL_NAVER                 = 433,
+  NDPI_PROTOCOL_SHEIN                 = 434,
+  NDPI_PROTOCOL_TEMU                  = 435,
+  NDPI_PROTOCOL_TAOBAO                = 436,
+  NDPI_PROTOCOL_MIKROTIK              = 437,
+  NDPI_PROTOCOL_DICOM                 = 438, /* Hospital Protocol */
+  NDPI_PROTOCOL_PARAMOUNTPLUS         = 439,
+  NDPI_PROTOCOL_YANDEX_ALICE          = 440,
 
 #ifdef CUSTOM_NDPI_PROTOCOLS
 #include "../../../nDPI-custom/custom_ndpi_protocol_ids.h"
