@@ -33,6 +33,7 @@
 using namespace std;
 
 bool ndAddr::Create(ndAddr &a, const string &addr) {
+    a.Reset();
     string _addr(addr);
 
     size_t p;
@@ -109,6 +110,7 @@ bool ndAddr::Create(ndAddr &a, const string &addr) {
 }
 
 bool ndAddr::Create(ndAddr &a, const uint8_t *hw_addr, size_t length) {
+    a.Reset();
     switch (length) {
     case ETH_ALEN:
 #if defined(__linux__)
@@ -136,6 +138,7 @@ bool ndAddr::Create(ndAddr &a, const uint8_t *hw_addr, size_t length) {
 bool ndAddr::Create(ndAddr &a,
   const struct sockaddr_storage *ss_addr,
   uint8_t prefix) {
+    a.Reset();
     switch (ss_addr->ss_family) {
     case AF_INET:
         if (prefix > _ND_ADDR_BITSv4) {
@@ -174,6 +177,7 @@ bool ndAddr::Create(ndAddr &a,
 
 bool ndAddr::Create(ndAddr &a,
   const struct sockaddr_in *ss_in, uint8_t prefix) {
+    a.Reset();
     if (ss_in->sin_family != AF_INET) {
         nd_dprintf("Unsupported address family: %hu\n",
           ss_in->sin_family);
@@ -196,6 +200,7 @@ bool ndAddr::Create(ndAddr &a,
 
 bool ndAddr::Create(ndAddr &a,
   const struct sockaddr_in6 *ss_in6, uint8_t prefix) {
+    a.Reset();
     if (ss_in6->sin6_family != AF_INET6) {
         nd_dprintf("Unsupported address family: %hu\n",
           ss_in6->sin6_family);
@@ -218,6 +223,7 @@ bool ndAddr::Create(ndAddr &a,
 
 bool ndAddr::Create(ndAddr &a,
   const struct in_addr *in_addr, uint8_t prefix) {
+    a.Reset();
     if (prefix > _ND_ADDR_BITSv4) {
         nd_dprintf(
           "Invalid IP address prefix length: %hhu\n", prefix);
@@ -236,6 +242,7 @@ bool ndAddr::Create(ndAddr &a,
 
 bool ndAddr::Create(ndAddr &a,
   const struct in6_addr *in6_addr, uint8_t prefix) {
+    a.Reset();
     if (prefix > _ND_ADDR_BITSv6) {
         nd_dprintf(
           "Invalid IP address prefix length: %hhu\n", prefix);
@@ -254,10 +261,16 @@ bool ndAddr::Create(ndAddr &a,
 
 const uint8_t *ndAddr::GetAddress(void) const {
     if (! IsValid()) return nullptr;
-    if (IsIPv4()) return (const uint8_t *)&addr.in.sin_addr;
+    if (IsIPv4())
+        return reinterpret_cast<const uint8_t *>(&addr.in.sin_addr);
     if (IsIPv6())
-        return (const uint8_t *)&addr.in6.sin6_addr;
-
+        return reinterpret_cast<const uint8_t *>(&addr.in6.sin6_addr);
+    if (IsEthernet())
+#if defined(__linux__)
+        return reinterpret_cast<const uint8_t *>(&addr.ll.sll_addr[0]);
+#elif defined(__FreeBSD__)
+        return reinterpret_cast<const uint8_t *>(&addr.dl.sdl_data[0]);
+#endif
     return nullptr;
 }
 
@@ -265,6 +278,7 @@ size_t ndAddr::GetAddressSize(void) const {
     if (! IsValid()) return 0;
     if (IsIPv4()) return sizeof(struct in_addr);
     if (IsIPv6()) return sizeof(struct in6_addr);
+    if (IsEthernet()) return ETH_ALEN;
 
     return 0;
 }
@@ -419,6 +433,82 @@ bool ndAddr::MakeString(const ndAddr &a, string &result,
     return false;
 }
 
+bool ndAddr::operator==(const ndAddr &a) const {
+    if (a.addr.ss.ss_family != addr.ss.ss_family)
+        return false;
+    if (ndFlagBoolean(compare_flags, CompareFlags::PREFIX) &&
+      a.prefix != prefix)
+        return false;
+
+    switch (addr.ss.ss_family) {
+#if defined(__linux__)
+    case AF_PACKET:
+        if (! ndFlagBoolean(compare_flags, CompareFlags::ADDR))
+            return true;
+        return (memcmp(&addr.ll, &a.addr.ll,
+                  sizeof(struct sockaddr_ll)) == 0);
+#elif defined(__FreeBSD__)
+    case AF_LINK:
+        if (! ndFlagBoolean(compare_flags, CompareFlags::ADDR))
+            return true;
+        return (memcmp(&addr.dl, &a.addr.dl,
+                  sizeof(struct sockaddr_dl)) == 0);
+#endif
+    case AF_INET:
+        if (ndFlagBoolean(compare_flags, CompareFlags::ADDR) &&
+          (ndFlagBoolean(compare_flags, CompareFlags::PORT)))
+        {
+            return (memcmp(&addr.in, &a.addr.in,
+                      sizeof(struct sockaddr_in)) == 0);
+        }
+        if (ndFlagBoolean(compare_flags, CompareFlags::ADDR) &&
+          memcmp(&addr.in.sin_addr, &a.addr.in.sin_addr,
+            sizeof(struct in_addr)) == 0)
+            return true;
+        if (ndFlagBoolean(compare_flags, CompareFlags::PORT) &&
+          addr.in.sin_port == a.addr.in.sin_port)
+            return true;
+        break;
+    case AF_INET6:
+        if (ndFlagBoolean(compare_flags, CompareFlags::ADDR) &&
+          (ndFlagBoolean(compare_flags, CompareFlags::PORT)))
+        {
+            return (memcmp(&addr.in6, &a.addr.in6,
+                      sizeof(struct sockaddr_in6)) == 0);
+        }
+        if (ndFlagBoolean(compare_flags, CompareFlags::ADDR) &&
+          memcmp(&addr.in6.sin6_addr,
+            &a.addr.in6.sin6_addr,
+            sizeof(struct in6_addr)) == 0)
+            return true;
+        if (ndFlagBoolean(compare_flags, CompareFlags::PORT) &&
+          addr.in6.sin6_port == a.addr.in6.sin6_port)
+            return true;
+        break;
+    }
+    return false;
+}
+
+int ndAddr::Compare(const ndAddr &a) const {
+    switch (addr.ss.ss_family) {
+#if defined(__linux__)
+    case AF_PACKET:
+        return memcmp(&addr.ll, &a.addr.ll, sizeof(struct sockaddr_ll));
+#elif defined(__FreeBSD__)
+    case AF_LINK:
+        return memcmp(&addr.dl, &a.addr.dl, sizeof(struct sockaddr_dl));
+#endif
+    case AF_INET:
+        return memcmp(
+            &addr.in.sin_addr, &a.addr.in.sin_addr, sizeof(struct in_addr));
+    case AF_INET6:
+        return memcmp(
+            &addr.in6.sin6_addr, &a.addr.in6.sin6_addr, sizeof(struct in6_addr));
+        break;
+    }
+    return 0;
+}
+
 ndAddrLookup::ndAddrLookup() {
     // Add private networks
     AddAddress(ndAddr::Type::RESERVED, "127.0.0.0/8");
@@ -453,10 +543,10 @@ bool ndAddrLookup::AddAddress(ndAddr::Type type,
       (! ifname.empty()) ? ifname.c_str() : "(global)",
       addr.GetString(ndAddr::MakeFlags::PREFIX).c_str());
 #endif
-    lock_guard<mutex> ul(lock);
-
     try {
         if (addr.IsEthernet()) {
+            lock_guard<mutex> ul(lock_ether_reserved);
+
             string mac = addr.GetString(ndAddr::MakeFlags::PREFIX);
             auto it = ether_reserved.find(mac);
             if (it != ether_reserved.end()) {
@@ -467,8 +557,10 @@ bool ndAddrLookup::AddAddress(ndAddr::Type type,
                 return false;
             }
             ether_reserved[mac] = type;
+#ifdef _ND_ENABLE_DEBUG_STATS
             nd_dprintf("ether_reserved: %u\n",
               ether_reserved.size());
+#endif
             return true;
         }
 
@@ -479,6 +571,8 @@ bool ndAddrLookup::AddAddress(ndAddr::Type type,
             ndRadixNetworkEntry<_ND_ADDR_BITSv4> entry;
             if (ndRadixNetworkEntry<_ND_ADDR_BITSv4>::Create(entry, addr))
             {
+                lock_guard<mutex> ul(lock_ipv4_reserved);
+
                 ipv4_reserved[entry] = type;
                 return true;
             }
@@ -488,6 +582,8 @@ bool ndAddrLookup::AddAddress(ndAddr::Type type,
             ndRadixNetworkEntry<_ND_ADDR_BITSv6> entry;
             if (ndRadixNetworkEntry<_ND_ADDR_BITSv6>::Create(entry, addr))
             {
+                lock_guard<mutex> ul(lock_ipv6_reserved);
+
                 ipv6_reserved[entry] = type;
                 return true;
             }
@@ -497,7 +593,9 @@ bool ndAddrLookup::AddAddress(ndAddr::Type type,
             ndRadixNetworkEntry<_ND_ADDR_BITSv4> entry;
             if (ndRadixNetworkEntry<_ND_ADDR_BITSv4>::Create(entry, addr))
             {
-                ipv4_iface[ifname][entry] = type;
+                lock_guard<mutex> ul(lock_ipv4_ifaces);
+
+                ipv4_ifaces[ifname][entry] = type;
                 return true;
             }
         }
@@ -506,7 +604,9 @@ bool ndAddrLookup::AddAddress(ndAddr::Type type,
             ndRadixNetworkEntry<_ND_ADDR_BITSv6> entry;
             if (ndRadixNetworkEntry<_ND_ADDR_BITSv6>::Create(entry, addr))
             {
-                ipv6_iface[ifname][entry] = type;
+                lock_guard<mutex> ul(lock_ipv6_ifaces);
+
+                ipv6_ifaces[ifname][entry] = type;
                 return true;
             }
         }
@@ -532,10 +632,10 @@ bool ndAddrLookup::RemoveAddress(const ndAddr &addr,
       (! ifname.empty()) ? ifname.c_str() : "(global)",
       addr.GetString(ndAddr::MakeFlags::PREFIX).c_str());
 #endif
-    lock_guard<mutex> ul(lock);
-
     try {
         if (addr.IsEthernet()) {
+            lock_guard<mutex> ul(lock_ether_reserved);
+
             string mac = addr.GetString(ndAddr::MakeFlags::PREFIX);
             auto it = ether_reserved.find(mac);
             if (it != ether_reserved.end()) {
@@ -549,6 +649,8 @@ bool ndAddrLookup::RemoveAddress(const ndAddr &addr,
             ndRadixNetworkEntry<_ND_ADDR_BITSv4> entry;
             if (ndRadixNetworkEntry<_ND_ADDR_BITSv4>::Create(entry, addr))
             {
+                lock_guard<mutex> ul(lock_ipv4_reserved);
+
                 return ipv4_reserved.erase(entry);
             }
         }
@@ -557,6 +659,8 @@ bool ndAddrLookup::RemoveAddress(const ndAddr &addr,
             ndRadixNetworkEntry<_ND_ADDR_BITSv6> entry;
             if (ndRadixNetworkEntry<_ND_ADDR_BITSv6>::Create(entry, addr))
             {
+                lock_guard<mutex> ul(lock_ipv6_reserved);
+
                 return ipv6_reserved.erase(entry);
             }
         }
@@ -565,8 +669,10 @@ bool ndAddrLookup::RemoveAddress(const ndAddr &addr,
             ndRadixNetworkEntry<_ND_ADDR_BITSv4> entry;
             if (ndRadixNetworkEntry<_ND_ADDR_BITSv4>::Create(entry, addr))
             {
-                auto it = ipv4_iface.find(ifname);
-                if (it != ipv4_iface.end())
+                lock_guard<mutex> ul(lock_ipv4_ifaces);
+
+                auto it = ipv4_ifaces.find(ifname);
+                if (it != ipv4_ifaces.end())
                     return it->second.erase(entry);
                 return false;
             }
@@ -576,8 +682,10 @@ bool ndAddrLookup::RemoveAddress(const ndAddr &addr,
             ndRadixNetworkEntry<_ND_ADDR_BITSv6> entry;
             if (ndRadixNetworkEntry<_ND_ADDR_BITSv6>::Create(entry, addr))
             {
-                auto it = ipv6_iface.find(ifname);
-                if (it != ipv6_iface.end())
+                lock_guard<mutex> ul(lock_ipv6_ifaces);
+
+                auto it = ipv6_ifaces.find(ifname);
+                if (it != ipv6_ifaces.end())
                     return it->second.erase(entry);
                 return false;
             }
@@ -592,6 +700,165 @@ bool ndAddrLookup::RemoveAddress(const ndAddr &addr,
     return false;
 }
 
+bool ndAddrLookup::AddLinkAddress(
+    unsigned index, const ndAddr &addr) {
+    if (! (addr.IsValid() && addr.IsEthernet())) return false;
+
+    lock_guard<mutex> ul(lock_link_addrs);
+
+    auto result = link_addrs.insert(make_pair(index, addr));
+
+    return result.second;
+}
+
+bool ndAddrLookup::RemoveLinkAddress(unsigned index) {
+    lock_guard<mutex> ul(lock_link_addrs);
+
+    auto result = link_addrs.find(index);
+    if (result == link_addrs.end()) return false;
+
+    link_addrs.erase(result);
+
+    return true;
+}
+
+bool ndAddrLookup::LookupLinkAddress(unsigned index, ndAddr &addr) {
+    lock_guard<mutex> ul(lock_link_addrs);
+
+    auto result = link_addrs.find(index);
+    if (result == link_addrs.end()) return false;
+
+    addr = result->second;
+
+    return true;
+}
+
+bool ndAddrLookup::AddLinkVLANId(unsigned index, uint16_t id) {
+    lock_guard<mutex> ul(lock_link_vlan_id);
+
+    auto result = link_vlan_ids.insert(make_pair(index, id));
+
+    return result.second;
+}
+
+bool ndAddrLookup::RemoveLinkVLANId(unsigned index) {
+    lock_guard<mutex> ul(lock_link_vlan_id);
+
+    auto result = link_vlan_ids.find(index);
+    if (result == link_vlan_ids.end()) return false;
+
+    link_vlan_ids.erase(result);
+
+    return true;
+}
+
+uint16_t ndAddrLookup::LookupLinkVLANId(unsigned index) {
+    lock_guard<mutex> ul(lock_link_vlan_id);
+
+    auto result = link_vlan_ids.find(index);
+    if (result == link_vlan_ids.end()) return 0;
+
+    return result->second;
+}
+
+bool ndAddrLookup::AddLinkName(
+    unsigned index, const string &name) {
+
+    lock_guard<mutex> ul(lock_link_names);
+
+    auto result = link_names.insert(make_pair(index, name));
+
+    return result.second;
+}
+
+bool ndAddrLookup::RemoveLinkName(unsigned index) {
+    lock_guard<mutex> ul(lock_link_names);
+
+    auto result = link_names.find(index);
+    if (result == link_names.end()) return false;
+
+    link_names.erase(result);
+
+    return true;
+}
+
+bool ndAddrLookup::LookupLinkName(unsigned index, string &name) {
+    lock_guard<mutex> ul(lock_link_names);
+
+    auto result = link_names.find(index);
+    if (result == link_names.end()) return false;
+
+    name = result->second;
+
+    return true;
+}
+
+bool ndAddrLookup::AddNeighbor(const ndAddr &addr, const ndAddr &lladdr) {
+    if (! (addr.IsValid() && lladdr.IsValid())) return false;
+    if (! (addr.IsIP() && lladdr.IsEthernet())) return false;
+
+    lock_guard<mutex> ul(lock_neighbors);
+
+    auto result = neighbors.insert(make_pair(addr, lladdr));
+
+    return result.second;
+#if 0
+    if (result.second) return true;
+
+    if (result.first->second != lladdr) {
+        result.first->second = lladdr;
+        nd_dprintf("%s: neighbor updated.\n", __PRETTY_FUNCTION__);
+        return true;
+    }
+
+    nd_dprintf("%s: neighbor already up-to-date.\n", __PRETTY_FUNCTION__);
+
+    return true;
+#endif
+}
+
+bool ndAddrLookup::RemoveNeighbor(const ndAddr &addr) {
+    if (! addr.IsValid()) return false;
+    if (! (addr.IsIP() || addr.IsEthernet())) return false;
+
+    lock_guard<mutex> ul(lock_neighbors);
+
+    bool removed = false;
+
+    if (addr.IsIP()) {
+        auto result = neighbors.find(addr);
+        if (result == neighbors.end()) return false;
+
+        neighbors.erase(result);
+        removed = true;
+    }
+    else {
+        for (neighbor_map::iterator i = neighbors.begin();
+            i != neighbors.end(); i++) {
+            if (i->second != addr) continue;
+
+            neighbors.erase(i);
+            removed = true;
+            break;
+        }
+    }
+
+    return removed;
+}
+
+bool ndAddrLookup::LookupNeighbor(const ndAddr &addr, ndAddr &lladdr) {
+    if (! (addr.IsValid() && addr.IsIP())) return false;
+
+    lock_guard<mutex> ul(lock_neighbors);
+
+    auto result = neighbors.find(addr);
+    if (result == neighbors.end()) return false;
+
+    lladdr = result->second;
+
+    return true;
+}
+
 void ndAddrLookup::Classify(ndAddr::Type &type, const ndAddr &addr) {
     if (addr.IsValid()) type = ndAddr::Type::OTHER;
     else {
@@ -649,9 +916,9 @@ void ndAddrLookup::Classify(ndAddr::Type &type, const ndAddr &addr) {
             return;
         }
 #endif
-        if (ether_reserved.size()) {
-            lock_guard<mutex> ul(lock);
+        lock_guard<mutex> ul(lock_ether_reserved);
 
+        if (ether_reserved.size()) {
             auto it = ether_reserved.find(addr.GetString());
             if (it != ether_reserved.end()) {
                 type = it->second;
@@ -670,13 +937,14 @@ void ndAddrLookup::Classify(ndAddr::Type &type, const ndAddr &addr) {
             return;
         }
 
-        for (auto &iface : ipv4_iface) {
+        lock_guard<mutex> ul_ipv4_ifaces(lock_ipv4_ifaces);
+        lock_guard<mutex> ul_ipv4_reserved(lock_ipv4_reserved);
+
+        for (auto &iface : ipv4_ifaces) {
             ndRadixNetworkEntry<_ND_ADDR_BITSv4> entry;
             if (ndRadixNetworkEntry<_ND_ADDR_BITSv4>::CreateQuery(
                   entry, addr))
             {
-                lock_guard<mutex> ul(lock);
-
                 nd_rn4_atype::iterator it;
                 if ((it = iface.second.longest_match(entry)) !=
                   iface.second.end())
@@ -694,8 +962,6 @@ void ndAddrLookup::Classify(ndAddr::Type &type, const ndAddr &addr) {
         if (ndRadixNetworkEntry<_ND_ADDR_BITSv4>::CreateQuery(
               entry, addr))
         {
-            lock_guard<mutex> ul(lock);
-
             nd_rn4_atype::iterator it;
             if ((it = ipv4_reserved.longest_match(entry)) !=
               ipv4_reserved.end())
@@ -709,13 +975,14 @@ void ndAddrLookup::Classify(ndAddr::Type &type, const ndAddr &addr) {
         }
     }
     else if (addr.IsIPv6()) {
-        for (auto &iface : ipv6_iface) {
+        lock_guard<mutex> ul_ipv6_ifaces(lock_ipv6_ifaces);
+        lock_guard<mutex> ul_ipv6_reserved(lock_ipv6_reserved);
+
+        for (auto &iface : ipv6_ifaces) {
             ndRadixNetworkEntry<_ND_ADDR_BITSv6> entry;
             if (ndRadixNetworkEntry<_ND_ADDR_BITSv6>::CreateQuery(
                   entry, addr))
             {
-                lock_guard<mutex> ul(lock);
-
                 nd_rn6_atype::iterator it;
                 if ((it = iface.second.longest_match(entry)) !=
                   iface.second.end())
@@ -733,8 +1000,6 @@ void ndAddrLookup::Classify(ndAddr::Type &type, const ndAddr &addr) {
         if (ndRadixNetworkEntry<_ND_ADDR_BITSv6>::CreateQuery(
               entry, addr))
         {
-            lock_guard<mutex> ul(lock);
-
             nd_rn6_atype::iterator it;
             if ((it = ipv6_reserved.longest_match(entry)) !=
               ipv6_reserved.end())
@@ -751,11 +1016,12 @@ void ndAddrLookup::Classify(ndAddr::Type &type, const ndAddr &addr) {
 
 size_t ndAddrLookup::GetInterfaceAddresses(const string &iface,
   set<string> &result, sa_family_t family) {
-    lock_guard<mutex> ul(lock);
 
     if (family == AF_UNSPEC || family == AF_INET) {
-        auto rn = ipv4_iface.find(iface);
-        if (rn == ipv4_iface.end()) return result.size();
+        lock_guard<mutex> ul(lock_ipv4_ifaces);
+
+        auto rn = ipv4_ifaces.find(iface);
+        if (rn == ipv4_ifaces.end()) return result.size();
 
         for (auto &it : rn->second) {
             string ip;
@@ -764,8 +1030,10 @@ size_t ndAddrLookup::GetInterfaceAddresses(const string &iface,
     }
 
     if (family == AF_UNSPEC || family == AF_INET6) {
-        auto rn = ipv6_iface.find(iface);
-        if (rn == ipv6_iface.end()) return result.size();
+        lock_guard<mutex> ul(lock_ipv6_ifaces);
+
+        auto rn = ipv6_ifaces.find(iface);
+        if (rn == ipv6_ifaces.end()) return result.size();
 
         for (auto &it : rn->second) {
             string ip;
